{***********************************************************************
** LogiLyzer: Steuerprogramm fr die gleichnamige Platine TN20190328, **
** die per RS232 & USB-Adapter vom PC gesteuert wird. Entwickelt mit  **
** Delphi3, getestet auf Windows 7, (c) Dipl.-Ing. Thomas Neveling    **
************************************************************************

Es werden nur Delphi3 Standard-Komponenten und die UNIT RS232 verwendet.
}
UNIT Unit1;

INTERFACE

USES
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, Mask, ExtCtrls, RS232;

TYPE  Str8 = STRING [8];

TYPE
  TForm1 = CLASS (TForm)
    ButtonPanel : TPanel;
    SampleRate : TComboBox;
    SampleLable : TLabel;
    TriggerMskLabel : TLabel;
    TriggerMask : TMaskEdit;
    Startsampling : TSpeedButton;
    PaintBox1 : TPaintBox;
    CH0 : TLabel;
    CH1 : TLabel;
    CH3 : TLabel;
    CH2 : TLabel;
    CH4 : TLabel;
    CH5 : TLabel;
    CH6 : TLabel;
    CH7 : TLabel;
    BtnStore : TSpeedButton;
    BtnLoad : TSpeedButton;
    OpenDialog1 : TOpenDialog;
    SaveDialog1 : TSaveDialog;
    C1Time : TEdit;
    C2Time : TEdit;
    DeltaTime : TEdit;
    ScrollBar1 : TScrollBar;
    SRStart : TEdit;
    SREnd : TEdit;
    ZoomX4 : TCheckBox;
    LED1 : TShape;
    TPos : TEdit;
    C1Byte : TEdit;
    C2Byte : TEdit;
    C1Chr : TEdit;
    C2Chr : TEdit;
    Marker1 : TGroupBox;
    Marker2 : TGroupBox;
    Status :  TGroupBox;
    PaintBox2 : TPaintBox;
    Analog : TLabel;
    Freq: TEdit;
    PROCEDURE FormCreate (Sender : TObject);
    PROCEDURE StartsamplingClick (Sender : TObject);
    PROCEDURE PaintBox1Paint (Sender : TObject);
    PROCEDURE BtnStoreClick (Sender : TObject);
    PROCEDURE BtnLoadClick (Sender : TObject);
    PROCEDURE PaintBox1MouseDown (Sender : TObject; Button : TMouseButton;
                                  Shift : TShiftState; X, Y : INTEGER);
    PROCEDURE ScrollBar1Change (Sender : TObject);
    PROCEDURE ZoomX4Click (Sender : TObject);
    PROCEDURE TriggerMaskExit (Sender : TObject);
    PROCEDURE TriggerMaskEnter (Sender : TObject);
  PRIVATE
    PROCEDURE ShowData (first, last : INTEGER);
    PROCEDURE CalcTime;
    FUNCTION GetByteValue (x : INTEGER) : Str8;
    FUNCTION GetChr (x : INTEGER) : CHAR;
  PUBLIC
  END;

VAR
  Form1 : TForm1;

IMPLEMENTATION

{$R *.DFM}

CONST BuffSize  = 2000;

VAR   RAMImage  : ARRAY [0..BuffSize - 1] OF BYTE;  { Puffer fr Sample-Bytes }
      Frst, Lst : INTEGER;     { 1. / letztes anzuzeigendes Byte aus RAMImage }
      SampFile  : FILE OF BYTE;{ Laden & Speichern des RAMImages }
      C1x, C2x  : INTEGER;     { Cursorlinien fr Zeitmessungen  }
      TrigPos   : INTEGER;     { Nr. des Triggerbytes im SRAM    }

FUNCTION TForm1.GetChr (x : INTEGER) : CHAR;
BEGIN { TForm1.GetChr }
 IF ZoomX4.Checked THEN Result:= Chr (RAMImage [Frst + x DIV 12])
                   ELSE Result:= Chr (RAMImage [Frst + x DIV 3])
END { TForm1.GetChr };

FUNCTION TForm1.GetByteValue (x : INTEGER) : Str8;
VAR      b, i, m : BYTE; s : Str8;
BEGIN { TForm1.GetByteValue }
 IF ZoomX4.Checked THEN b:= RAMImage [Frst + x DIV 12]
                   ELSE b:= RAMImage [Frst + x DIV 3];
 s:= '';
 m:= 128;
 FOR i:= 1 TO 8 DO
  BEGIN
   IF b AND m > 0 THEN s:= s + '1'
                  ELSE s:= s + '0';
   m:= m SHR 1
  END { for i };
 Result:= s
END { TForm1.GetByteValue };

PROCEDURE TForm1.CalcTime;
VAR       t1, t2, Delta : LongInt;
          St            : INTEGER;
BEGIN { TForm1.CalcTime }
 St:= 1;                       { Defaultwert fr Testmode  }
 CASE SampleRate.ItemIndex OF
  0 : St:= 1;                  { Sampling Intervalle in s }
  1 : St:= 2;
  2 : St:= 5;
  3 : St:= 10;
  4 : St:= 20;
  5 : St:= 50;
  6 : St:= 100;
  7 : St:= 200;
  8 : St:= 500;
  9 : St:= 1000;
  10 : St:= 2000;
  11 : St:= 4000;
  12 : St:= 8000;
  13 : St:= 16000
 END { case SampleRate };
 IF ZoomX4.Checked THEN
  BEGIN
   t1:= Round (C1x * St / 12);
   t2:= Round (C2x * St / 12)
  END { if ZoomX4 }
 ELSE
  BEGIN
   t1:= Round (C1x * St / 3);
   t2:= Round (C2x * St / 3)
  END { else  };
 C1Time.Text:= IntToStr (t1) + ' s';
 C2Time.Text:= IntToStr (t2) + ' s';
 Delta:= Abs (t2 - t1);
 DeltaTime.Text:= IntToStr (Delta) + ' s';
 TPos.Text:= 'Trigger ' + IntToStr (TrigPos);
 C1Byte.Text:= GetByteValue (C1x);
 C2Byte.Text:= GetByteValue (C2x);
 C1Chr.Text := GetChr (C1x);
 C2Chr.Text := GetChr (C2x);
 IF Delta > 0 THEN
    Freq.Text:= IntToStr (1000000 DIV Delta) + ' Hz'
END { TForm1.CalcTime };

PROCEDURE TForm1.ShowData (first, last : INTEGER); { Diagramm zeichnen     }
VAR       n, Wert      : BYTE;
          m, XStep, tp : INTEGER;
BEGIN { TForm1.ShowData }
 IF ZoomX4.Checked THEN xstep:= 12 ELSE xstep:= 3; { Pixel je 0/1 bergang }

 WITH PaintBox1.Canvas DO
  BEGIN
   Pen.Color  := ClLtGray;                         { Zeichenflche anlegen }
   Brush.Color:= ClBlack;
   Rectangle (0, 0, 1344, 197);

   Pen.Color:= ClYellow;
   Wert:= 1;         { Bitmaske: Bit0 }
   FOR n:= 1 TO 8 DO                              { Signalverlauf zeichnen }
    BEGIN
     FOR m:= first TO last DO                     { Traceabstand: 24 Pixel }
      BEGIN                                       { "1" = 12 Pixel hoch    }
       MoveTo ((m - first)   * xstep, n * 24 - 12 * ((RAMImage [m - 1] AND Wert) DIV Wert));
       LineTo ((m - first)   * xstep, n * 24 - 12 * ((RAMImage [m] AND Wert) DIV Wert));
       LineTo ((m - first + 1) * xstep, n * 24 - 12 * ((RAMImage [m] AND Wert) DIV Wert))
      END { for m };
     Wert:= Wert * 2 { nchst hheres Bit }
    END { for n };

   Pen.Color:= clAqua;         { Cursorlinien einfgen }
   MoveTo (C1x, 0);  LineTo (C1x, 196);
   Pen.Color:= clLime;
   MoveTo (C2x, 0);  LineTo (C2x, 196);
   CalcTime;

   Pen.Color:= clRed;          { Triggermarke setzen }
   IF (TrigPos >= First) AND (TrigPos <= Last) THEN
    BEGIN
     tp:= TrigPos - First;
     MoveTo (tp * xstep, 0);
     LineTo (tp * xstep, 196)
    END { if TrigPos }
  END { with PaintBox1 };

  WITH PaintBox2.Canvas DO    { Analog-Ausgabe (Summenwerte) }
  BEGIN
   Pen.Color  := ClLtGray;
   Brush.Color:= ClBlack;
   Rectangle (0, 0, 1344, 135);
   Pen.Color:= clFuchsia;

   MoveTo (0, 1 + (255 - RAMImage [first]) DIV 2);
   FOR m:= first TO last DO
    BEGIN
     LineTo ((m - first)   * xstep, 1 + (255 - RAMImage [m - 1]) DIV 2);
     LineTo ((m - first)   * xstep, 1 + (255 - RAMImage [m])   DIV 2);
     LineTo ((m - first + 1) * xstep, 1 + (255 - RAMImage [m])   DIV 2)
    END { for m }
  END { with PaintBox2 }
END { TForm1.ShowData };

PROCEDURE TForm1.FormCreate (Sender : TObject);
VAR       s : STRING;
BEGIN { Initialisierungsteil }
 SampleRate.ItemIndex:= 0;
 FillChar (RamImage, SizeOf (RamImage), $00);
 s:= InputBox ('COM-Port auswhlen..', '1..99 ohne fhrende Null', 'COM6:19200,N,8,1');
 IF OPENCOM (Pchar (s)) <> 1 THEN Caption:= Caption + ' | disconnected'
 ELSE
  BEGIN
   Caption:= Caption + ' <=> ' + s;
   StartSampling.Enabled:= TRUE;
   TIMEOUTS (80)
  END { else };
 Frst:= 0;
 Lst := 447;
 C1x := 15;
 C2x := 30
END { TForm1.FormCreate };

FUNCTION CalcTrigger (s : Str8) : INTEGER;      { Binrstring => Integer }
VAR      Trig : INTEGER;
         m, i : BYTE;
BEGIN { CalcTrigger }
 m:= 128;
 Trig:= 0;
 FOR i:= 1 TO 8 DO
  BEGIN
   IF (s [i] = '1') THEN Trig:= Trig OR m;
   m:= m SHR 1
  END { for i };
 Result:= Trig
END { CalcTrigger };

PROCEDURE TForm1.StartsamplingClick (Sender : TObject);
VAR       Count, TxByte, RxByte : INTEGER;
BEGIN { TForm1.StartsamplingClick }
 TxByte:= SampleRate.ItemIndex;
 SendByte (TxByte);        { Code fr Samplerate: 0..14 }
 TxByte:= CalcTrigger (TriggerMask.Text);
 SendByte (TxByte);
 LED1.Brush.Color:= clLime; LED1.Refresh; { LED: Sample }

 REPEAT                        { auf Sendebeginn warten }
  RxByte:= ReadByte
 UNTIL RxByte <> - 1;
 LED1.Brush.Color:= clRed; LED1.Refresh;    { LED: RxD  }

 TrigPos:= RxByte;                          { Low Byte  }
 TrigPos:= TrigPos + 256 * ReadByte;        { High Byte }
 TrigPos:= TrigPos - 256;                   { SRAM offs }

 FOR Count:= 0 TO BuffSize - 1 DO    { restliche Byte lesen und abspeichern }
     RamImage [Count]:= ReadByte;

 LED1.Brush.Color:= clOlive; LED1.Refresh;  { LED: Idle }
 ShowData (Frst, Lst)
END { TForm1.StartsamplingClick };

PROCEDURE TForm1.PaintBox1Paint (Sender : TObject);
BEGIN { TForm1.PaintBox1Paint }
 ShowData (Frst, Lst)
END { TForm1.PaintBox1Paint };

PROCEDURE TForm1.BtnStoreClick (Sender : TObject);
VAR       i : INTEGER;
          r : BYTE;
BEGIN { TForm1.BtnStoreClick }
 IF SaveDialog1.Execute THEN
  BEGIN
   AssignFile (SampFile, SaveDialog1.Filename);
   Rewrite (SampFile);
   r:= SampleRate.ItemIndex; Write (SampFile, r);
   r:= TrigPos SHR 8;        Write (SampFile, r);  { Trigger High }
   r:= TrigPos AND $00FF;    Write (SampFile, r);  { Trigger Low  }
   FOR i:= 0 TO BuffSize - 1 DO Write (SampFile, RAMImage [i]);
   CloseFile (SampFile)
  END { if SaveDialog1 }
END { TForm1.BtnStoreClick };

PROCEDURE TForm1.BtnLoadClick (Sender : TObject);
VAR       i : INTEGER;
          r : BYTE;
BEGIN { TForm1.BtnLoadClick }
 IF OpenDialog1.Execute THEN
  BEGIN
   AssignFile (SampFile, OpenDialog1.FileName);
   Reset (SampFile);
   Read (SampFile, r); SampleRate.ItemIndex:= r;
   Read (SampFile, r); TrigPos:= r * 256;
   Read (SampFile, r); TrigPos:= TrigPos + r;
   FOR i:= 0 TO BuffSize - 1 DO Read (SampFile, RAMImage [i]);
   CloseFile (SampFile);
   ShowData (Frst, Lst)
  END { if OpenDialog1 }
END { TForm1.BtnLoadClick };

PROCEDURE TForm1.PaintBox1MouseDown (Sender : TObject; Button : TMouseButton;
                                     Shift : TShiftState; X, Y : INTEGER);
BEGIN { TForm1.PaintBox1MouseDown }
 IF Button = mbRight THEN C1x:= x ELSE C2x:= x; { Marker Positionen }
 ShowData (Frst, Lst)
END { TForm1.PaintBox1MouseDown };

PROCEDURE TForm1.ScrollBar1Change (Sender : TObject);
BEGIN { TForm1.ScrollBar1Change }
 Frst:= ScrollBar1.Position;
 IF ZoomX4.Checked THEN
  BEGIN
   Lst:= Frst + 111;
   Scrollbar1.Max:= 1888 { 1888 + 111 = 1999 }
  END { if ZoomX4 }
 ELSE
  BEGIN
   Lst:= Frst + 447;
   Scrollbar1.Max:= 1552 { 1552 + 447 = 1999 }
  END { else  };
 SRStart.Text:= IntToStr (Frst);
 SREnd.Text  := IntToStr (Lst);
 ShowData (Frst, Lst)
END { TForm1.ScrollBar1Change };

PROCEDURE TForm1.ZoomX4Click (Sender : TObject);
BEGIN { TForm1.ZoomX4Click }
 ScrollBar1Change (self)
END { TForm1.ZoomX4Click };

VAR MaskPattern : Str8;  { Backup Triggerbyte }

FUNCTION ValidBin (S : Str8) : BOOLEAN;
CONST    BinZiffern  : SET OF CHAR = ['0'..'1'];
VAR      i           : BYTE;
BEGIN { ValidBin }
 Result:= TRUE;
 FOR i:= 1 TO 8 DO IF NOT (S [i] IN BinZiffern) THEN Result:= FALSE;
 IF Result = FALSE THEN ShowMessage ('kein Binrwert!')
END { ValidBin };

PROCEDURE TForm1.TriggerMaskExit (Sender : TObject);
BEGIN { TForm1.TriggerMaskExit }
 IF NOT ValidBin (TriggerMask.Text) THEN TriggerMask.Text:= MaskPattern
END { TForm1.TriggerMaskExit };

PROCEDURE TForm1.TriggerMaskEnter (Sender : TObject);
BEGIN { TForm1.TriggerMaskEnter }
 MaskPattern:= TriggerMask.Text
END { TForm1.TriggerMaskEnter };

END { UNIT: Unit1.pas }.
